#- Requires several R packages (most importantly, hdrm houses the data needed to run this analysis)

rm(list = ls())
library(data.table)
library(dplyr)
library(hdrm)
library(glmnet)
library(xrnet)
library(doParallel)
library(pROC)
library(VennDiagram)

dat <- readData(bcTCGA)
X   <- cbind(dat$y, dat$X)
X   <- scale(X, scale = TRUE)
colnames(X)[1] <- "BRCA1"
variantId <- colnames(X)

tau.seq <- seq(0.05, 0.95, by = 0.05)
nCores <- detectCores() - 2; nCores

y.idx <- which(variantId == "BRCA1")


set.seed(2021)
train.idx <- sample(1:dim(X)[1], dim(X)[1] * 2/3)
foldId <- sample(1:10, length(train.idx), replace = TRUE)

# Fit LASSO
registerDoParallel(nCores)
fit.lasso  <- cv.glmnet(X[train.idx, -y.idx], X[train.idx, y.idx], 
                        family = "gaussian",                      
                        alpha = 1,
                        type.measure = "deviance",
                        nfolds = 10,
                        foldid = foldId,
                        nlambda = 20,
                        parallel = TRUE)
stopImplicitCluster()

l1_beta <- coef(fit.lasso, s = "lambda.min")[-1]

# Non-negative LASSO
registerDoParallel(nCores)
fit.nlasso  <- cv.glmnet(X[train.idx, -y.idx], X[train.idx, y.idx], 
                         family = "gaussian",                      
                         alpha = 1,
                         lower.limits = 0,
                         type.measure = "deviance",
                         nfolds = 10,
                         foldid = foldId,
                         nlambda = 20,
                         parallel = TRUE)
stopImplicitCluster()

nl1_beta <- coef(fit.nlasso, s = "lambda.min")[-1]


# Non-positive LASSO
registerDoParallel(nCores)
fit.nlasso2  <- cv.glmnet(X[train.idx, -y.idx], X[train.idx, y.idx], 
                          family = "gaussian",                      
                          alpha = 1,
                          upper.limits = 0,
                          type.measure = "deviance",
                          nfolds = 10,
                          foldid = foldId,
                          nlambda = 20,
                          parallel = TRUE)
stopImplicitCluster()

nl1_beta2 <- coef(fit.nlasso2, s = "lambda.min")[-1]


# Fit asymmetric LASSO
quantile_beta <- matrix(NA, nrow = length(tau.seq), 
                        ncol = (dim(X)[2] - 1))
quantile_cvm <- quantile_sd <- quantile_b0 <- numeric()
rownames(quantile_beta) <- paste("tau = ", tau.seq)

for(j in 1:length(tau.seq)) {
  #cat(paste(j))
  registerDoParallel(nCores)
  fit.quantile       <- tune_xrnet(X[train.idx, -y.idx], X[train.idx, y.idx], 
                                   family = "gaussian",
                                   penalty_main = define_penalty("q1", tau.seq[j], 
                                                                 num_penalty = 20),
                                   loss = "deviance",
                                   nfolds = 10,
                                   foldid = foldId,
                                   parallel = TRUE)
  stopImplicitCluster()
  quantile_b0[j] <- coef(fit.quantile)$beta0
  quantile_beta[j, ] <- coef(fit.quantile)$betas[, , 1]
  quantile_cvm[j]    <- min(fit.quantile$cv_mean)
  quantile_sd[j] <- fit.quantile$cv_sd[which.min(fit.quantile$cv_mean)]
}


q1_beta <- quantile_beta[which.min(quantile_cvm), ]
q1_tau <- which.min(quantile_cvm)

# Figure 4
plot(quantile_cvm, ylab = "Mean Cross Validation Error",
     xlab = expression(tau), xaxt = "n", type = "o", pch = 19)
axis(1, at = seq(2, 19, by = 2), labels = tau.seq[seq(2, 19, by = 2)])
min(fit.nlasso$cvm)


# Get linear predictors to calculated predicted R2
l1_eta <- cbind(1, X[-train.idx, -y.idx]) %*% coef(fit.lasso, s = "lambda.min")
nl1_eta <- cbind(1, X[-train.idx, -y.idx]) %*% coef(fit.nlasso, s = "lambda.min")
nl1_eta2 <- cbind(1, X[-train.idx, -y.idx]) %*% coef(fit.nlasso2, s = "lambda.min")

q1_eta <- quantile_b0[which.min(quantile_cvm)] + X[-train.idx, -y.idx] %*% q1_beta


y.test <- X[-train.idx, y.idx]

# Table 6 
1 - sum((y.test - l1_eta)^2) / sum((y.test - mean(y.test))^2) # LASSO R2
1 - sum((y.test - nl1_eta)^2) / sum((y.test - mean(y.test))^2) # Non-negative LASSO R2
1 - sum((y.test - nl1_eta2)^2) / sum((y.test - mean(y.test))^2) # Non-positive LASSO R2
1 - sum((y.test - q1_eta)^2) / sum((y.test - mean(y.test))^2) # asymLASSO R2


# No. of positive effects
sum(l1_beta > 0) # LASSO
sum(nl1_beta > 0) # Non-negative LASSO
sum(nl1_beta2 > 0) # Non-positive LASSO
sum(q1_beta > 0) # asymLASSO


# No. of negative effects
sum(l1_beta <  0) # LASSO
sum(nl1_beta < 0) # Non-negative LASSO
sum(nl1_beta2 < 0) # Non-postiive LASSO
sum(q1_beta < 0) # asymLASSO

