#- Simulation 1: Logistic Regression (Q1 vs. LASSO)
rm(list = ls())
library(xrnet)
library(readxl)
library(glmnet)
library(pROC)
library(doParallel)

#- Source important files
source("sourceFiles/sourceFiles.R")
source("sourceFiles/getLossFunctions.R")
simSheet <- read_xlsx("simulationList.xlsx", sheet = 2)
simSheet
B <- 100

tau.seq <- seq(0.05, 0.95, by = 0.05)
nCores  <- 6 
for(sim in 1:4) {
  #- Simulation parameters
  simParams <- simSheet[sim, ]
  n.train   <- simParams$nobs
  n.cov     <- simParams$ncovs
  b0        <- c(0.08, 0, 0, 0.10, 0, 0.12, 0.15, 0, 0, 0.25)
  
  #- Change signal signs here
  if (simParams$sign == 2) {
    b0 <- b0 * c(-1, 0, 0, 1, 0, 1, 1, 0, 0, -1)
  } 
  
  beta0 <- c(b0, rep(0, n.cov - length(b0)))
  
  n.sim <- n.train + 1000
  V     <- createCorrelationStructure(p = n.cov, method = "ar", rho = 0.50)
  cV     <- chol(V)
  out <- list() # Results from simulation
  set.seed(simParams$seed)
  for(b in 1:B) {
    cat(paste0("Simulation: ", b, "\r"))
    #- Simulate data
    Z   <- matrix(rnorm(n.sim * n.cov), nrow = n.sim)
    Z   <- Z %*% cV # Create correlation structurte
    
    #- Simulate Logistic Regression Data
    intcpt <- 0.5
    y      <- rbinom(n.sim, 1, 1 / (1 + exp(-(intcpt + Z %*% beta0))))   
    
    #- Split training and test set    
    y.train <- y[1:n.train]
    Z.train <- Z[1:n.train, ]
    y.test  <- y[-(1:n.train)]
    Z.test  <- Z[-(1:n.train), ]
    
    #- Fit ORACLE model
    fit.oracle  <- glm(y.train ~ Z.train[, which(beta0 != 0)], 
                       family = "binomial")
    oracle_beta <- numeric(n.cov)
    oracle_beta[which(beta0 != 0)] <- fit.oracle$coefficients[-1]
    pred.oracle <- fit.oracle$coefficients[1]  + Z.test[, which(beta0 != 0)] %*% fit.oracle$coefficients[-1]
    oracle.res  <- as.vector(exp(pred.oracle) / (1 + exp(pred.oracle)))
    oracle_mse  <- auc(y.test, oracle.res, quiet = TRUE)[1]
    #- ncvreg fits
    
    #- Create Fold ID for Cross Validation
    foldId <- sample(1:5, size = length(y.train), replace = TRUE)
    
    #- LASSO
    registerDoParallel(nCores)
    fit.lasso  <- cv.glmnet(Z.train, y.train, family = "binomial", 
                            alpha = 1,
                            type.measure = "deviance",
                            nfolds = 5,
                            lambda.min.ratio = 0.5,
                            foldid = foldId,
                            nlambda = 20,
                            parallel = TRUE)
    stopImplicitCluster()
    
    lasso_beta <- coef(fit.lasso, s = "lambda.min")[-1]
    pred.lasso <- predict(fit.lasso, newx = Z.test, 
                          s = "lambda.min", type = "response")
    lasso_mse  <- auc(y.test, as.vector(pred.lasso), quiet = TRUE)[1]
    
    #- Fit Q1 regularization 
    quantile_beta <- matrix(NA, nrow = length(tau.seq), ncol = n.cov)
    rownames(quantile_beta) <- paste("tau = ", tau.seq)
    
    quantile_cvm  <- numeric()
    quantile_mse  <- numeric()
    quantile_r2   <- numeric()
    
    for(j in 1:length(tau.seq)) {
      registerDoParallel(nCores)
      fit.quantile       <- tune_xrnet(Z.train, y.train, family = "binomial",
                                       penalty_main = define_penalty("q1", tau.seq[j], 
                                                                     num_penalty = 20,
                                                                     penalty_ratio = 0.5),
                                       loss = "deviance",
                                       nfolds = 5,
                                       foldid = foldId,
                                       parallel = TRUE)
      stopImplicitCluster()
      
      quantile_beta[j, ] <- coef(fit.quantile)$betas[, , 1]
      quantile_cvm[j]    <- min(fit.quantile$cv_mean)
      pred.quantile      <- predict(fit.quantile, newdata = Z.test,
                                    p = "opt")
      quantile_mse[j]    <- auc(y.test, as.vector(pred.quantile), quiet = TRUE)
    }
    
    #- Concatenate results:
    results_from_simulation <- list()
    results_from_simulation$betas <- rbind(oracle_beta, #1
                                           lasso_beta, #2
                                           quantile_beta[which.min(quantile_cvm), ], 
                                           quantile_beta)
    results_from_simulation$opt   <- c(tau.seq[which.min(quantile_cvm)])
    results_from_simulation$mse   <- c(oracle_mse,
                                       lasso_mse, 
                                       quantile_mse[which.min(quantile_cvm)],
                                       quantile_mse)
    results_from_simulation$meanY <- mean(y)
    out[[b]] <- results_from_simulation
  } # End simulation
  # Save simulation results
  save(simParams, out, beta0, file = paste0("results/results_s3_HD_ar_", simParams$model, ".RData"))
}
