#- Simulation 4: Sign dependence

rm(list = ls())
library(xrnet)
library(readxl)
library(glmnet)
library(pROC)
library(doParallel)

#- Source important files
source("sourceFiles/sourceFiles.R")
source("sourceFiles/getLossFunctions.R")
simSheet <- read_xlsx("simulationList.xlsx", sheet = 3)
simSheet
B <- 100

tau.seq <- seq(0.05, 0.95, by = 0.05)
nCores <- 6
for(sim in c(2, 3)) {
  #- Simulation parameters
  simParams <- simSheet[sim, ]
  n.train   <- simParams$nobs
  n.cov     <- 10
  b0        <- c(-0.03, 0, 0, -0.03, 0, 0.05, 0.05, 0, 0, 0.08) 
  
  beta0 <- c(b0, rep(0, n.cov - length(b0)))
  #ext   <- c(1, 0, 0, 1, 1, 0, 0, 1, 0, 1, rep(0, n.cov - length(b0)))
  
  n.sim <- n.train + 1000
  V     <- createCorrelationStructure(p = n.cov, method = "ar", rho = simParams$rho)
  cV    <- chol(V)
  out   <- list() # Results from simulation
  
  set.seed(simParams$seed)
  for(b in 1:B) {
    cat(paste0("Simulation: ", b, "\r"))
    #- Simulate data
    Z   <- matrix(rnorm(n.sim * n.cov), nrow = n.sim)
    Z   <- Z %*% cV # Create correlation structurte
    
    intcpt <- 0.10
    y      <- rnorm(n.sim, mean = intcpt + Z %*% beta0,
                    sd = 0.5)   
    
    #- Split training and test set    
    y.train <- y[1:n.train]
    Z.train <- Z[1:n.train, ]
    
    Z2.train      <- Z.train
    Z2.train[, 1] <- -Z2.train[, 1]
    Z2.train[, 4] <- -Z2.train[, 4]
    

    #- Create Fold ID for Cross Validation
    foldId <- sample(1:5, size = length(y.train), replace = TRUE)
    
    #- LASSO
    registerDoParallel(nCores)
    fit.lasso1  <- cv.glmnet(Z.train, y.train, family = "gaussian", 
                            alpha = 1,
                            type.measure = "deviance",
                            nfolds = 5,
                            foldid = foldId,
                            nlambda = 20,
                            parallel = TRUE)
    stopImplicitCluster()
    
    registerDoParallel(nCores)
    fit.lasso2  <- cv.glmnet(Z2.train, y.train, family = "gaussian", 
                             alpha = 1,
                             type.measure = "deviance",
                             nfolds = 5,
                             foldid = foldId,
                             nlambda = 20,
                             parallel = TRUE)
    stopImplicitCluster()
    
    
    
    lasso1_beta <- coef(fit.lasso1, s = "lambda.min")[-1]
    lasso2_beta <- coef(fit.lasso2, s = "lambda.min")[-1]
    
    #- nLASSO
    registerDoParallel(nCores)
    fit.lasso1  <- cv.glmnet(Z.train, y.train, family = "gaussian", 
                             alpha = 1,
                             type.measure = "deviance",
                             lower.limit = 0,
                             nfolds = 5,
                             foldid = foldId,
                             nlambda = 20,
                             parallel = TRUE)
    stopImplicitCluster()
    
    registerDoParallel(nCores)
    fit.lasso2  <- cv.glmnet(Z2.train, y.train, family = "gaussian", 
                             alpha = 1,
                             type.measure = "deviance",
                             nfolds = 5,
                             foldid = foldId,
                             lower.limit = 0,
                             nlambda = 20,
                             parallel = TRUE)
    stopImplicitCluster()
    
    
    
    nlasso1_beta <- coef(fit.lasso1, s = "lambda.min")[-1]
    nlasso2_beta <- coef(fit.lasso2, s = "lambda.min")[-1]
    
    
    #- Fit Q1 regularization 
    quantile_beta1 <- matrix(NA, nrow = length(tau.seq), ncol = n.cov)
    quantile_beta2 <- matrix(NA, nrow = length(tau.seq), ncol = n.cov)
    
    quantile1_cvm  <- numeric()
    quantile2_cvm  <- numeric()
    
    for(j in 1:length(tau.seq)) {
      registerDoParallel(nCores)
      fit.quantile1       <- tune_xrnet(Z.train, y.train, family = "gaussian",
                                       penalty_main = define_penalty("q1", tau.seq[j], 
                                                                     num_penalty = 20),
                                       loss = "deviance",
                                       nfolds = 5,
                                       foldid = foldId,
                                       parallel = TRUE)
      stopImplicitCluster()
      
      registerDoParallel(nCores)
      fit.quantile2       <- tune_xrnet(Z2.train, y.train, family = "gaussian",
                                        penalty_main = define_penalty("q1", tau.seq[j], 
                                                                      num_penalty = 20),
                                        loss = "deviance",
                                        nfolds = 5,
                                        foldid = foldId,
                                        parallel = TRUE)
      stopImplicitCluster()
      
      
      quantile_beta1[j, ] <- coef(fit.quantile1)$betas[, , 1]
      quantile_beta2[j, ] <- coef(fit.quantile2)$betas[, , 1]
      
      quantile1_cvm[j]    <- min(fit.quantile1$cv_mean)
      quantile2_cvm[j]    <- min(fit.quantile2$cv_mean)
      
      }
    
    #- Concatenate results:
    results_from_simulation <- list()
    results_from_simulation$betas <- rbind(lasso1_beta, #2
                                           lasso2_beta,
                                           nlasso1_beta,
                                           nlasso2_beta,
                                           quantile_beta1[which.min(quantile1_cvm), ], 
                                           quantile_beta2[which.min(quantile2_cvm), ])
    results_from_simulation$opt   <- c(tau.seq[which.min(quantile1_cvm)],
                                       tau.seq[which.min(quantile2_cvm)])
    
    out[[b]] <- results_from_simulation
  } # End simulation
  # Save simulation results
  save(simParams, out, beta0, file = paste0("results/results_s3_ar_", simParams$model, ".RData"))
}